/*
Programa con el PIC12F629 ver 2, que detecta la seal infrarroja de un control
remoto MITZU o clon y se basa en detectar el pulso start, luego el guarda-banda
y determinar si es un bit 0 o bit 1, de ADR, ADRC y en seguida hacer lo mismo
con CMD y CMDC. La seal est a nivel bajo antes de comenzar a presentarse, ya 
que es invertida desde el sensor IR por un transistor. Utiliza un cristal de 12
MHz para la exactitud de detecciones. Se utiliza un control remoto MITZU MRC-ZN4
con las teclas POWER para encender y para apagar (una sola tecla) y codigo 3,
as como la tecla CH- para encender 4 LEDs que se montan en el socket del foco
y son excitados por ra2 a travs de un BC548 usando el cdigo 7.

Alfredo Segura, Queretaro Mexico, agosto de 2019
*/
//------------------------------------------------------------------------------
#include <12F629.h>
#fuses HS,NOWDT,NOPROTECT,PUT,NOMCLR,NOBROWNOUT

#use delay(clock = 12000000)
#use fast_io(A)

#byte porta = 0x05

#bit  ra0 = 0x05.0      //Salida activar transistor del Relay
#bit  ra1 = 0x05.1      //
#bit  ra2 = 0x05.2      //Salida activar LES blancos. Alto activo.
#bit  ra3 = 0x05.3      //Entrada Seal infrarrojo
#bit  ra4 = 0x05.4      //XTAL IN
#bit  ra5 = 0x05.5      //XTAL OUT

#rom 0x03FF={0x3438}    //solo poner si se usa el USP 3.0 de VOLNIA

//---------------------------VARIABLES-------------------------------------
int i = 0, j = 0, m = 0;
int adr = 0, adrc = 0,  cmd = 0, cmdc = 0;
int bait, nbait, vari;
int1 bandera = 0, bit;
long Tm5 = 274, T = 548, T2 = 1096;  //my bueno para el control remoto MITZU 1096

//-------------------------INVIERTE---------------------------------------------
void invierte (vari) {
for (i=0; i<=7; i++) {
bait = vari;
j = 7 - i;
bit = bit_test(bait,i);    //prueba bit 0 de bait y lo pasa al bit 7 de nbait
if (bit == 1) {
bit_set(nbait,j);
}
if (bit == 0) {
bit_clear(nbait,j);
  }
 }
}
//---------------------------------INI----------------------------------------
void ini(void) {
    set_tris_a(0b11001000);   //Salidas datos BCD y relay y entrada pulsos.
    porta = 0x00;             //LEDs blancos apagados
 }
//-----------------------RUTINA PRINCIPAL-------------------------------------- 
void main(void)   {
  ini();
  delay_ms(100);
  ra0 = 1;
  ra2 = 0;
While (true) {
if (ra3 == 0) {
goto EXIT;
}
if (ra3 == 1) {         //Inicia START al ir a nivel alto   ___/   \___

while (ra3 == 1) {      //Aqui espera a que el pulso START transcurra alto
//espera que transcurra START unos 8926 us
j++;
}                       //Sale de START y entra a SYNC
while (ra3 == 0) {
//espera que transcurra SYNC unos 4407 us en bajo        \____
j++;
}                       //Sale de SYNC cuando el pulso va a ALTO

cmd = 0; adr = 0;             //Borra cualquier contenido anterior de adr y cmd
cmdc = 0; adrc = 0;

delay_us(Tm5);          //Espera 0.5T antes de comenzar     ____/

for (i=0; i<=7;i++) {   //Comienza por detectar adr
delay_us(T2);           //Espera 2T y decide el valor del primer bit
if (ra3 == 1) {         //Lee el GB siguiente al primer bit. Ley un 1. el bit es 0
bit = 0;
adr = (adr<<1) & 0xFE;  //el bit es "0"
}
if (ra3 == 0) {         //Aqui no ley el GB, sino el valle de un bit 1.
adr = (adr<<1) | 0x01;           //el bit es "1"
bit = 1;
   }
if (bit == 1) {         //Esperar un tiempo adicional de 2T
delay_us(T2);
}
//                      //Si bit = 0, no pone tiempo adicional
}
//
for (i=0; i<=7;i++) {   //sigue detectando adrc
delay_us(T2);           //Espera 2T y decide el valor del primer bit
if (ra3 == 1) {         //Lee el GB siguiente al primer bit. Ley un 1. el bit es 0
bit = 0;
adrc = (adrc<<1) & 0xFE;  //el bit es "0"
}
if (ra3 == 0) {         //Aqui no ley el GB, sino el valle de un bit 1.
adrc = (adrc<<1) | 0x01;           //el bit es "1"
bit = 1;
   }
if (bit == 1) {         //Esperar un tiempo adicional de 2T
delay_us(T2);
}
//                      //Si bit = 0, no pone tiempo adicional
}
//
for (i=0; i<=7;i++) {   //sigue detectando cmd
delay_us(T2);           //Espera 2T y decide el valor del primer bit
if (ra3 == 1) {         //Lee el GB siguiente al primer bit. Ley un 1. el bit es 0
bit = 0;
cmd = (cmd<<1) & 0xFE;  //el bit es "0"
}
if (ra3 == 0) {         //Aqui no ley el GB, sino el valle de un bit 1.
cmd = (cmd<<1) | 0x01;           //el bit es "1"
bit = 1;
   }
if (bit == 1) {         //Esperar un tiempo adicional de 2T
delay_us(T2);
}
//                      //Si bit = 0, no pone tiempo adicional
}
//
for (i=0; i<=7;i++) {   //Finaliza dtectando cmdc
delay_us(T2);           //Espera 2T y decide el valor del primer bit
if (ra3 == 1) {         //Lee el GB siguiente al primer bit. Ley un 1. el bit es 0
bit = 0;
bit_clear(cmdc,(7-i));
//cmdc = (cmdc<<1) & 0xFE;  //el bit es "0"
}
if (ra3 == 0) {         //Aqui no ley el GB, sino el valle de un bit 1.
bit_set(cmdc,(7-i));
//cmdc = (cmdc<<1) | 0x01;           //el bit es "1"
bit = 1;
   }
if (bit == 1) {         //Esperar un tiempo adicional de 2T
delay_us(T2);
}
//                      //Si bit = 0, no pone tiempo adicional
}
//------- Ahora invierte la posicin de los bits LSB a la derecha de cada byte:----------
//invierte (adr);
//adr = nbait;
//invierte (adrc);
//adrc = nbait;
invierte (cmd);
cmd = nbait;
invierte (cmdc);
cmdc = nbait;
bandera = 1;            //Indica datos listos...
}

EXIT:
delay_us(890);

if (bandera == 1) {
   if (cmdc == 3) {      //Tecla POWER Mitzu1 (codigo 923)
      ra0++;               //Valor 3 para Mitzu2 Codigo 0221)
      ra2 = 0;
      delay_ms(800);
    bandera = 0;
}
   if (cmdc == 7) {      //Tecla CH- Mitzu1 (codigo 923)
      ra2++;               //Valor 7 para Mitzu2, codigo 0221
      ra0 = 0;
      delay_ms(800);
    bandera = 0;
}
m++;
}
  }// Se cierra While
}//Se cierra Main
//------------------------------------------------------------------------------
